@@ -62,6 +62,7 @@ class SimplePOP3Retriever(POP3RetrieverBase, POP3initMixIn):
         ConfInt(name='port', required=False, default=110),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
+        ConfTupleOfStrings(name='password_command', required=False, default=()),
         ConfBool(name='use_apop', required=False, default=False),
         ConfBool(name='delete_dup_msgids', required=False, default=False),
     )
@@ -95,6 +96,7 @@ class SimplePOP3SSLRetriever(POP3RetrieverBase, POP3SSLinitMixIn):
         ConfInt(name='port', required=False, default=POP3_ssl_port),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
+        ConfTupleOfStrings(name='password_command', required=False, default=()),
         ConfBool(name='use_apop', required=False, default=False),
         ConfBool(name='delete_dup_msgids', required=False, default=False),
         ConfFile(name='keyfile', required=False, default=None),
@@ -103,6 +105,7 @@ class SimplePOP3SSLRetriever(POP3RetrieverBase, POP3SSLinitMixIn):
         ConfTupleOfStrings(name='ssl_fingerprints', required=False, default=()),
         ConfString(name='ssl_version', required=False, default=None),
         ConfString(name='ssl_ciphers', required=False, default=None),
+        ConfString(name='ssl_cert_hostname', required=False, default=None),
     )
     received_from = None
     received_with = 'POP3-SSL'
@@ -171,6 +174,7 @@ class BrokenUIDLPOP3Retriever(BrokenUIDLPOP3RetrieverBase, POP3initMixIn):
         ConfInt(name='port', required=False, default=110),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
+        ConfTupleOfStrings(name='password_command', required=False, default=()),
         ConfBool(name='use_apop', required=False, default=False),
     )
     received_with = 'POP3'
@@ -201,6 +205,7 @@ class BrokenUIDLPOP3SSLRetriever(BrokenUIDLPOP3RetrieverBase, POP3SSLinitMixIn):
         ConfInt(name='port', required=False, default=POP3_ssl_port),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
+        ConfTupleOfStrings(name='password_command', required=False, default=()),
         ConfBool(name='use_apop', required=False, default=False),
         ConfFile(name='keyfile', required=False, default=None),
         ConfFile(name='certfile', required=False, default=None),
@@ -208,6 +213,7 @@ class BrokenUIDLPOP3SSLRetriever(BrokenUIDLPOP3RetrieverBase, POP3SSLinitMixIn):
         ConfTupleOfStrings(name='ssl_fingerprints', required=False, default=()),
         ConfString(name='ssl_version', required=False, default=None),
         ConfString(name='ssl_ciphers', required=False, default=None),
+        ConfString(name='ssl_cert_hostname', required=False, default=None),
     )
     received_with = 'POP3-SSL'
 
@@ -237,6 +243,7 @@ class MultidropPOP3Retriever(MultidropPOP3RetrieverBase, POP3initMixIn):
         ConfInt(name='port', required=False, default=110),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
+        ConfTupleOfStrings(name='password_command', required=False, default=()),
         ConfBool(name='use_apop', required=False, default=False),
         ConfString(name='envelope_recipient'),
     )
@@ -270,6 +277,7 @@ class MultidropPOP3SSLRetriever(MultidropPOP3RetrieverBase, POP3SSLinitMixIn):
         ConfInt(name='port', required=False, default=POP3_ssl_port),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
+        ConfTupleOfStrings(name='password_command', required=False, default=()),
         ConfBool(name='use_apop', required=False, default=False),
         ConfString(name='envelope_recipient'),
         ConfFile(name='keyfile', required=False, default=None),
@@ -278,6 +286,7 @@ class MultidropPOP3SSLRetriever(MultidropPOP3RetrieverBase, POP3SSLinitMixIn):
         ConfTupleOfStrings(name='ssl_fingerprints', required=False, default=()),
         ConfString(name='ssl_version', required=False, default=None),
         ConfString(name='ssl_ciphers', required=False, default=None),
+        ConfString(name='ssl_cert_hostname', required=False, default=None),
     )
     received_from = None
     received_with = 'POP3-SSL'
@@ -316,6 +325,7 @@ class MultidropSDPSRetriever(SimplePOP3Retriever, POP3initMixIn):
         ConfInt(name='port', required=False, default=110),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
+        ConfTupleOfStrings(name='password_command', required=False, default=()),
         # Demon apparently doesn't support APOP
         ConfBool(name='use_apop', required=False, default=False),
     )
@@ -368,10 +378,12 @@ class SimpleIMAPRetriever(IMAPRetrieverBase, IMAPinitMixIn):
         ConfInt(name='port', required=False, default=imaplib.IMAP4_PORT),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
+        ConfTupleOfStrings(name='password_command', required=False, default=()),
         ConfTupleOfUnicode(name='mailboxes', required=False,
                            default="('INBOX', )", allow_specials=('ALL',)),
         ConfBool(name='use_peek', required=False, default=True),
         ConfString(name='move_on_delete', required=False, default=None),
+        ConfBool(name='record_mailbox', required=False, default=True),
         # imaplib.IMAP4.login_cram_md5() requires the (unimplemented)
         # .authenticate(), so we can't do this yet (?).
         ConfBool(name='use_cram_md5', required=False, default=False),
@@ -402,16 +414,19 @@ class SimpleIMAPSSLRetriever(IMAPRetrieverBase, IMAPSSLinitMixIn):
         ConfInstance(name='configparser', required=False),
         ConfDirectory(name='getmaildir', required=False, default='~/.getmail/'),
 
-        # socket.ssl() and socket timeouts are incompatible in Python 2.3
-        #ConfInt(name='timeout', required=False, default=180),
+        # socket.ssl() and socket timeouts were incompatible in Python 2.3;
+        # if you have problems, comment this line out
+        ConfInt(name='timeout', required=False, default=180),
         ConfString(name='server'),
         ConfInt(name='port', required=False, default=imaplib.IMAP4_SSL_PORT),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
+        ConfTupleOfStrings(name='password_command', required=False, default=()),
         ConfTupleOfUnicode(name='mailboxes', required=False,
                            default="('INBOX', )", allow_specials=('ALL',)),
         ConfBool(name='use_peek', required=False, default=True),
         ConfString(name='move_on_delete', required=False, default=None),
+        ConfBool(name='record_mailbox', required=False, default=True),
         ConfFile(name='keyfile', required=False, default=None),
         ConfFile(name='certfile', required=False, default=None),
         ConfFile(name='ca_certs', required=False, default=None),
@@ -422,6 +437,7 @@ class SimpleIMAPSSLRetriever(IMAPRetrieverBase, IMAPSSLinitMixIn):
         # .authenticate(), so we can't do this yet (?).
         ConfBool(name='use_cram_md5', required=False, default=False),
         ConfBool(name='use_kerberos', required=False, default=False),
+        ConfString(name='ssl_cert_hostname', required=False, default=None),
     )
     received_from = None
     received_with = 'IMAP4-SSL'
@@ -453,10 +469,12 @@ class MultidropIMAPRetriever(MultidropIMAPRetrieverBase, IMAPinitMixIn):
         ConfInt(name='port', required=False, default=imaplib.IMAP4_PORT),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
+        ConfTupleOfStrings(name='password_command', required=False, default=()),
         ConfTupleOfUnicode(name='mailboxes', required=False,
                            default="('INBOX', )", allow_specials=('ALL',)),
         ConfBool(name='use_peek', required=False, default=True),
         ConfString(name='move_on_delete', required=False, default=None),
+        ConfBool(name='record_mailbox', required=False, default=True),
         # imaplib.IMAP4.login_cram_md5() requires the (unimplemented)
         # .authenticate(), so we can't do this yet (?).
         ConfBool(name='use_cram_md5', required=False, default=False),
@@ -488,16 +506,19 @@ class MultidropIMAPSSLRetriever(MultidropIMAPRetrieverBase, IMAPSSLinitMixIn):
         ConfInstance(name='configparser', required=False),
         ConfDirectory(name='getmaildir', required=False, default='~/.getmail/'),
 
-        # socket.ssl() and socket timeouts are incompatible in Python 2.3
-        #ConfInt(name='timeout', required=False, default=180),
+        # socket.ssl() and socket timeouts were incompatible in Python 2.3;
+        # if you have problems, comment this line out
+        ConfInt(name='timeout', required=False, default=180),
         ConfString(name='server'),
         ConfInt(name='port', required=False, default=imaplib.IMAP4_SSL_PORT),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
+        ConfTupleOfStrings(name='password_command', required=False, default=()),
         ConfTupleOfUnicode(name='mailboxes', required=False,
                            default="('INBOX', )", allow_specials=('ALL',)),
         ConfBool(name='use_peek', required=False, default=True),
         ConfString(name='move_on_delete', required=False, default=None),
+        ConfBool(name='record_mailbox', required=False, default=True),
         ConfFile(name='keyfile', required=False, default=None),
         ConfFile(name='certfile', required=False, default=None),
         ConfFile(name='ca_certs', required=False, default=None),
@@ -509,6 +530,7 @@ class MultidropIMAPSSLRetriever(MultidropIMAPRetrieverBase, IMAPSSLinitMixIn):
         ConfBool(name='use_cram_md5', required=False, default=False),
         ConfBool(name='use_kerberos', required=False, default=False),
         ConfString(name='envelope_recipient'),
+        ConfString(name='ssl_cert_hostname', required=False, default=None),
     )
     received_from = None
     received_with = 'IMAP4-SSL'
@@ -526,3 +548,4 @@ class MultidropIMAPSSLRetriever(MultidropIMAPRetrieverBase, IMAPSSLinitMixIn):
         self.log.trace()
         self.log.info('MultidropIMAPSSLRetriever(%s)' % self._confstring()
                       + os.linesep)
+
