@@ -6,7 +6,13 @@ __all__ = [
     'address_no_brackets',
     'change_usergroup',
     'change_uidgid',
+    'decode_crappy_text',
+    'format_header',
     'check_ssl_key_and_cert',
+    'check_ca_certs',
+    'check_ssl_version',
+    'check_ssl_fingerprints',
+    'check_ssl_ciphers',
     'deliver_maildir',
     'eval_bool',
     'expand_user_vars',
@@ -20,6 +26,8 @@ __all__ = [
     'gid_of_uid',
     'uid_of_user',
     'updatefile',
+    'get_password',
+    'run_command',
 ]
 
 
@@ -30,10 +38,39 @@ import signal
 import stat
 import time
 import glob
-
+import re
 import fcntl
 import pwd
 import grp
+import getpass
+import commands
+import sys
+import tempfile
+import errno
+try:
+    import subprocess
+except ImportError, o:
+    subprocess = None
+
+# hashlib only present in python2.5, ssl in python2.6; used together
+# in SSL functionality below
+try:
+    import ssl
+except ImportError:
+    ssl = None
+try:
+    import hashlib
+except ImportError:
+    hashlib = None
+
+# Optional gnome-keyring integration
+try:
+    import gnomekeyring
+    # And test to see if it's actually available
+    if not gnomekeyring.is_available():
+        gnomekeyring = None
+except ImportError:
+    gnomekeyring = None
 
 from getmailcore.exceptions import *
 
@@ -48,6 +85,8 @@ _bool_values = {
     'off'   : False,
     '0'     : False
 }
+osx_keychain_binary = '/usr/bin/security'
+
 
 #######################################
 def lock_file(file, locktype):
@@ -123,6 +162,7 @@ class updatefile(object):
         if self.closed or not hasattr(self, 'file'):
             return
         self.file.flush()
+        os.fsync(self.file.fileno())
         self.file.close()
         os.rename(self.tmpname, self.filename)
         self.closed = True
@@ -404,6 +444,29 @@ def change_uidgid(logger=None, uid=None, gid=None):
         raise getmailDeliveryError('change UID/GID to %s/%s failed (%s)'
                                    % (uid, gid, o))
 
+#######################################
+def decode_crappy_text(s):
+    '''Take a line of text in arbitrary and possibly broken bytestring encoding
+    and return an ASCII or unicode version of it.
+    '''
+    # first, assume it was written in the encoding of the user's terminal
+    lang = os.environ.get('LANG')
+    if lang:
+        try:
+            (lang, encoding) = lang.split('.')
+            return s.decode(encoding)
+        except (UnicodeError, ValueError), o:
+            pass
+    # that failed; try well-formed in various common encodings next
+    for encoding in ('ascii', 'utf-8', 'latin-1', 'utf-16'):
+        try:
+            return s.decode(encoding)
+        except UnicodeError, o:
+            continue
+    # all failed - force it
+    return s.decode('utf-8', 'replace')
+    
+
 #######################################
 def format_header(name, line):
     '''Take a long line and return rfc822-style multiline header.
@@ -468,3 +531,214 @@ def check_ssl_key_and_cert(conf):
             'optional certfile and keyfile must be supplied together'
         )
     return (keyfile, certfile)
+
+#######################################
+def check_ca_certs(conf):
+    ca_certs = conf['ca_certs']
+    if ca_certs is not None:
+        ca_certs = expand_user_vars(ca_certs)
+        if ssl is None:
+            raise getmailConfigurationError(
+                'specifying ca_certs not supported by this installation of '
+                'Python; requires Python 2.6'
+            )
+    if ca_certs and not os.path.isfile(ca_certs):
+        raise getmailConfigurationError(
+            'optional ca_certs must be path to a valid file'
+        )
+    return ca_certs
+
+#######################################
+def check_ssl_version(conf):
+    ssl_version = conf['ssl_version']
+    if ssl_version is None:
+        return None
+    if ssl is None:
+        raise getmailConfigurationError(
+            'specifying ssl_version not supported by this installation of '
+            'Python; requires Python 2.6'
+        )
+    ssl_version = ssl_version.lower()
+    if ssl_version == 'sslv23':
+        return ssl.PROTOCOL_SSLv23
+    elif ssl_version == 'sslv3':
+        return ssl.PROTOCOL_SSLv3
+    elif ssl_version == 'tlsv1':
+        return ssl.PROTOCOL_TLSv1
+    elif ssl_version == 'tlsv1_1' and 'PROTOCOL_TLSv1_1' in dir(ssl):
+        return ssl.PROTOCOL_TLSv1_1
+    elif ssl_version == 'tlsv1_2' and 'PROTOCOL_TLSv1_2' in dir(ssl):
+        return ssl.PROTOCOL_TLSv1_2
+    else:
+        raise getmailConfigurationError(
+            'unknown or unsupported ssl_version'
+        )
+
+#######################################
+def check_ssl_fingerprints(conf):
+    ssl_fingerprints = conf['ssl_fingerprints']
+    if ssl_fingerprints is ():
+        return ()
+    if ssl is None or hashlib is None:
+        raise getmailConfigurationError(
+            'specifying ssl_fingerprints not supported by this installation of '
+            'Python; requires Python 2.6'
+        )
+
+    normalized_fprs = []
+    for fpr in ssl_fingerprints:
+        fpr = fpr.lower().replace(':','')
+        if len(fpr) != 64:
+            raise getmailConfigurationError(
+                'ssl_fingerprints must each be the SHA256 certificate hash in hex (with or without colons)'
+            )
+        normalized_fprs.append(fpr)
+    return normalized_fprs
+
+#######################################
+def check_ssl_ciphers(conf):
+    ssl_ciphers = conf['ssl_ciphers']
+    if ssl_ciphers:
+        if sys.version_info < (2, 7, 0):
+            raise getmailConfigurationError(
+                'specifying ssl_ciphers not supported by this installation of '
+                'Python; requires Python 2.7'
+            )
+        if re.search(r'[^a-zA-z0-9, :!\-+@]', ssl_ciphers):
+            raise getmailConfigurationError(
+                'invalid character in ssl_ciphers'
+            )
+    return ssl_ciphers
+
+#######################################
+keychain_password = None
+if os.name == 'posix':
+    if os.path.isfile(osx_keychain_binary):
+        def keychain_password(user, server, protocol, logger):
+            """Mac OSX: return a keychain password, if it exists.  Otherwise, return
+         
+         None.
+            """
+            # OSX protocol is not an arbitrary string; it's a code limited to 
+            # 4 case-sensitive chars, and only specific values.
+            protocol = protocol.lower()
+            if 'imap' in protocol:
+                protocol = 'imap'
+            elif 'pop' in protocol:
+                protocol = 'pop3'
+            else:
+                # This will break.
+                protocol = '????'
+            
+            # wish we could pass along a comment to this thing for the user prompt
+            cmd = "%s find-internet-password -g -a '%s' -s '%s' -r '%s'" % (
+                osx_keychain_binary, user, server, protocol
+            )
+            (status, output) = commands.getstatusoutput(cmd)
+            if status != os.EX_OK or not output:
+                logger.error('keychain command %s failed: %s %s' 
+                             % (cmd, status, output))
+                return None
+            password = None
+            for line in output.split('\n'):
+                #match = re.match(r'password: "([^"]+)"', line)
+                #if match:
+                #    password = match.group(1)
+                if 'password:' in line:
+                    pw = line.split(':', 1)[1].strip()
+                    if pw.startswith('"') and pw.endswith('"'):
+                        pw = pw[1:-1]
+                    password = pw
+            if password is None:
+                logger.debug('No keychain password found for %s %s %s'
+                             % (user, server, protocol))
+            return password
+    elif gnomekeyring:
+        def keychain_password(user, server, protocol, logger):
+            """Gnome: return a keyring password, if it exists.  Otherwise, return
+            None.
+            """
+            #logger.trace('trying Gnome keyring for user="%s", server="%s", protocol="%s"\n'
+            #             % (user, server, protocol))
+            try:
+                # http://developer.gnome.org/gnome-keyring/3.5/gnome-keyring
+                # -Network-Passwords.html#gnome-keyring-find-network-password-sync
+                secret = gnomekeyring.find_network_password_sync(
+                    # user, domain=None, server, object=None, protocol,
+                    # authtype=None, port=0
+                    user, None, server, None, protocol, None, 0
+                )
+                
+                #logger.trace('got keyring result %s' % str(secret))
+            except gnomekeyring.NoMatchError:
+                logger.debug('gnome-keyring does not know password for %s %s %s'
+                             % (user, server, protocol))
+                return None
+
+            # secret looks like this:
+            # [{'protocol': 'imap', 'keyring': 'Default', 'server': 'gmail.com', 
+            #   'user': 'hiciu', 'item_id': 1L, 'password': 'kielbasa'}]
+            if secret and 'password' in secret[0]:
+                return secret[0]['password']
+
+            return None
+    #else:
+        # Posix but no OSX keychain or Gnome keyring.
+        # Fallthrough
+if keychain_password is None:
+    def keychain_password(user, server, protocol, logger):
+        """Neither Mac OSX keychain or Gnome keyring available: always return 
+        None.
+        """
+        return None
+
+
+#######################################
+def get_password(label, user, server, protocol, logger):
+    # try keychain/keyrings first, where available
+    password = keychain_password(user, server, protocol, logger)
+    if password:
+        logger.debug('using password from keychain/keyring')
+    else:
+        # no password found (or not on OSX), prompt in the usual way
+        password = getpass.getpass('Enter password for %s:  ' % label)
+    return password
+
+
+#######################################
+def run_command(command, args):
+    # Simple subprocess wrapper for running a command and fetching its exit 
+    # status and output/stderr.
+    if args is None:
+        args = []
+    if type(args) == tuple:
+        args = list(args)
+
+    # Programmer sanity checks
+    assert type(command) in (str, unicode), (
+        'command is %s (%s)' % (command, type(command))
+    )
+    assert type(args) == list, (
+        'args is %s (%s)' % (args, type(args))
+    )
+    for arg in args:
+        assert type(arg) in (str, unicode), 'arg is %s (%s)' % (arg, type(arg))
+
+    stdout = tempfile.TemporaryFile()
+    stderr = tempfile.TemporaryFile()
+
+    cmd = [command] + args
+
+    try:
+        p = subprocess.Popen(cmd, stdout=stdout, stderr=stderr)
+    except OSError, o:
+        if o.errno == errno.ENOENT:
+            # no such file, command not found
+            raise getmailConfigurationError('Program "%s" not found' % command)
+        #else:
+        raise
+
+    rc = p.wait()
+    stdout.seek(0)
+    stderr.seek(0)
+    return (rc, stdout.read().strip(), stderr.read().strip())
